@@ -11,13 +11,14 @@ use indicatif::{MultiProgress, ProgressBar, ProgressStyle};
 use path_slash::PathExt;
 use xwin::util::ProgressTarget;
 
+use crate::common::{CargoOptions, XWinOptions};
+
 /// Compile a local package and all of its dependencies
 #[derive(Clone, Debug, Default, Parser)]
 #[clap(setting = clap::AppSettings::DeriveDisplayOrder, after_help = "Run `cargo help build` for more detailed information.")]
 pub struct Build {
-    /// Do not print cargo log messages
-    #[clap(short = 'q', long)]
-    pub quiet: bool,
+    #[clap(flatten)]
+    pub cargo: CargoOptions,
 
     /// Package to build (see `cargo help pkgid`)
     #[clap(
@@ -40,10 +41,6 @@ pub struct Build {
     #[clap(long)]
     pub all: bool,
 
-    /// Number of parallel jobs, defaults to # of CPUs
-    #[clap(short = 'j', long, value_name = "N")]
-    pub jobs: Option<usize>,
-
     /// Build only this package's library
     #[clap(long)]
     pub lib: bool,
@@ -84,120 +81,20 @@ pub struct Build {
     #[clap(long)]
     pub all_targets: bool,
 
-    /// Build artifacts in release mode, with optimizations
-    #[clap(short = 'r', long)]
-    pub release: bool,
-
-    /// Build artifacts with the specified Cargo profile
-    #[clap(long, value_name = "PROFILE-NAME")]
-    pub profile: Option<String>,
-
-    /// Space or comma separated list of features to activate
-    #[clap(long, multiple_values = true)]
-    pub features: Vec<String>,
-
-    /// Activate all available features
-    #[clap(long)]
-    pub all_features: bool,
-
-    /// Do not activate the `default` feature
-    #[clap(long)]
-    pub no_default_features: bool,
-
-    /// Build for the target triple
-    #[clap(long, value_name = "TRIPLE", env = "CARGO_BUILD_TARGET")]
-    pub target: Option<String>,
-
-    /// Directory for all generated artifacts
-    #[clap(long, value_name = "DIRECTORY", parse(from_os_str))]
-    pub target_dir: Option<PathBuf>,
-
     /// Copy final artifacts to this directory (unstable)
     #[clap(long, value_name = "PATH", parse(from_os_str))]
     pub out_dir: Option<PathBuf>,
 
-    /// Path to Cargo.toml
-    #[clap(long, value_name = "PATH", parse(from_os_str))]
-    pub manifest_path: Option<PathBuf>,
-
-    /// Ignore `rust-version` specification in packages
-    #[clap(long)]
-    pub ignore_rust_version: bool,
-
-    /// Error format
-    #[clap(long, value_name = "FMT", multiple_values = true)]
-    pub message_format: Vec<String>,
-
     /// Output the build plan in JSON (unstable)
     #[clap(long)]
     pub build_plan: bool,
 
-    /// Output build graph in JSON (unstable)
-    #[clap(long)]
-    pub unit_graph: bool,
-
     /// Outputs a future incompatibility report at the end of the build (unstable)
     #[clap(long)]
     pub future_incompat_report: bool,
 
-    /// Use verbose output (-vv very verbose/build.rs output)
-    #[clap(short = 'v', long, parse(from_occurrences), max_occurrences = 2)]
-    pub verbose: usize,
-
-    /// Coloring: auto, always, never
-    #[clap(long, value_name = "WHEN")]
-    pub color: Option<String>,
-
-    /// Require Cargo.lock and cache are up to date
-    #[clap(long)]
-    pub frozen: bool,
-
-    /// Require Cargo.lock is up to date
-    #[clap(long)]
-    pub locked: bool,
-
-    /// Run without accessing the network
-    #[clap(long)]
-    pub offline: bool,
-
-    /// Override a configuration value (unstable)
-    #[clap(long, value_name = "KEY=VALUE", multiple_values = true)]
-    pub config: Vec<String>,
-
-    /// Unstable (nightly-only) flags to Cargo, see 'cargo -Z help' for details
-    #[clap(short = 'Z', value_name = "FLAG", multiple_values = true)]
-    pub unstable_flags: Vec<String>,
-
-    /// xwin cache directory
-    #[clap(long, parse(from_os_str), env = "XWIN_CACHE_DIR", hide = true)]
-    pub xwin_cache_dir: Option<PathBuf>,
-
-    /// The architectures to include in CRT/SDK
-    #[clap(
-        long,
-        env = "XWIN_ARCH",
-        possible_values(&["x86", "x86_64", "aarch", "aarch64"]),
-        use_value_delimiter = true,
-        default_value = "x86_64,aarch64",
-        hide = true,
-    )]
-    pub xwin_arch: Vec<xwin::Arch>,
-
-    /// The variants to include
-    #[clap(
-        long,
-        env = "XWIN_VARIANT",
-        possible_values(&["desktop", "onecore", /*"store",*/ "spectre"]),
-        use_value_delimiter = true,
-        default_value = "desktop",
-        hide = true,
-    )]
-    pub xwin_variant: Vec<xwin::Variant>,
-
-    /// The version to retrieve, can either be a major version of 15 or 16, or
-    /// a "<major>.<minor>" version.
-    #[clap(long, env = "XWIN_VERSION", default_value = "16", hide = true)]
-    pub xwin_version: String,
+    #[clap(flatten)]
+    pub xwin: XWinOptions,
 }
 
 impl Build {
@@ -214,7 +111,7 @@ impl Build {
 
     /// Generate cargo subcommand
     pub fn build_command(&self, subcommand: &str) -> Result<Command> {
-        let xwin_cache_dir = self.xwin_cache_dir.clone().unwrap_or_else(|| {
+        let xwin_cache_dir = self.xwin.xwin_cache_dir.clone().unwrap_or_else(|| {
             dirs::cache_dir()
                 // If the really is no cache dir, cwd will also do
                 .unwrap_or_else(|| env::current_dir().expect("Failed to get current dir"))
@@ -228,7 +125,7 @@ impl Build {
         build.arg(subcommand);
 
         // collect cargo build arguments
-        if self.quiet {
+        if self.cargo.quiet {
             build.arg("--quiet");
         }
         for pkg in &self.packages {
@@ -243,7 +140,7 @@ impl Build {
         if self.all {
             build.arg("--all");
         }
-        if let Some(jobs) = self.jobs {
+        if let Some(jobs) = self.cargo.jobs {
             build.arg("--jobs").arg(jobs.to_string());
         }
         if self.lib {
@@ -276,71 +173,71 @@ impl Build {
         if self.all_targets {
             build.arg("--all-targets");
         }
-        if self.release {
+        if self.cargo.release {
             build.arg("--release");
         }
-        if let Some(profile) = self.profile.as_ref() {
+        if let Some(profile) = self.cargo.profile.as_ref() {
             build.arg("--profile").arg(profile);
         }
-        for feature in &self.features {
+        for feature in &self.cargo.features {
             build.arg("--features").arg(feature);
         }
-        if self.all_features {
+        if self.cargo.all_features {
             build.arg("--all-features");
         }
-        if self.no_default_features {
+        if self.cargo.no_default_features {
             build.arg("--no-default-features");
         }
-        if let Some(target) = self.target.as_ref() {
+        if let Some(target) = self.cargo.target.as_ref() {
             build.arg("--target").arg(target);
         }
-        if let Some(dir) = self.target_dir.as_ref() {
+        if let Some(dir) = self.cargo.target_dir.as_ref() {
             build.arg("--target-dir").arg(dir);
         }
         if let Some(dir) = self.out_dir.as_ref() {
             build.arg("--out-dir").arg(dir);
         }
-        if let Some(path) = self.manifest_path.as_ref() {
+        if let Some(path) = self.cargo.manifest_path.as_ref() {
             build.arg("--manifest-path").arg(path);
         }
-        if self.ignore_rust_version {
+        if self.cargo.ignore_rust_version {
             build.arg("--ignore-rust-version");
         }
-        for fmt in &self.message_format {
+        for fmt in &self.cargo.message_format {
             build.arg("--message-format").arg(fmt);
         }
         if self.build_plan {
             build.arg("--build-plan");
         }
-        if self.unit_graph {
+        if self.cargo.unit_graph {
             build.arg("--unit-graph");
         }
         if self.future_incompat_report {
             build.arg("--future-incompat-report");
         }
-        if self.verbose > 0 {
-            build.arg(format!("-{}", "v".repeat(self.verbose)));
+        if self.cargo.verbose > 0 {
+            build.arg(format!("-{}", "v".repeat(self.cargo.verbose)));
         }
-        if let Some(color) = self.color.as_ref() {
+        if let Some(color) = self.cargo.color.as_ref() {
             build.arg("--color").arg(color);
         }
-        if self.frozen {
+        if self.cargo.frozen {
             build.arg("--frozen");
         }
-        if self.locked {
+        if self.cargo.locked {
             build.arg("--locked");
         }
-        if self.offline {
+        if self.cargo.offline {
             build.arg("--offline");
         }
-        for config in &self.config {
+        for config in &self.cargo.config {
             build.arg("--config").arg(config);
         }
-        for flag in &self.unstable_flags {
+        for flag in &self.cargo.unstable_flags {
             build.arg("-Z").arg(flag);
         }
 
-        if let Some(target) = self.target.as_ref() {
+        if let Some(target) = self.cargo.target.as_ref() {
             if target.contains("msvc") {
                 self.setup_msvc_crt(xwin_cache_dir.clone())?;
                 let env_target = target.to_lowercase().replace('-', "_");
@@ -417,6 +314,7 @@ impl Build {
     fn setup_msvc_crt(&self, cache_dir: PathBuf) -> Result<()> {
         let done_mark_file = cache_dir.join("DONE");
         let xwin_arches: HashSet<_> = self
+            .xwin
             .xwin_arch
             .iter()
             .map(|x| x.as_str().to_string())
@@ -439,10 +337,12 @@ impl Build {
         let pkg_manifest = self.load_manifest(&ctx, draw_target)?;
 
         let arches = self
+            .xwin
             .xwin_arch
             .iter()
             .fold(0, |acc, arch| acc | *arch as u32);
         let variants = self
+            .xwin
             .xwin_variant
             .iter()
             .fold(0, |acc, var| acc | *var as u32);
@@ -505,6 +405,7 @@ impl Build {
         ctx.execute(pkgs, work_items, arches, variants, op)?;
 
         let downloaded_arches: Vec<_> = self
+            .xwin
             .xwin_arch
             .iter()
             .map(|x| x.as_str().to_string())
@@ -538,8 +439,12 @@ impl Build {
         manifest_pb.set_prefix("Manifest");
         manifest_pb.set_message("📥 downloading");
 
-        let manifest =
-            xwin::manifest::get_manifest(ctx, &self.xwin_version, "release", manifest_pb.clone())?;
+        let manifest = xwin::manifest::get_manifest(
+            ctx,
+            &self.xwin.xwin_version,
+            "release",
+            manifest_pb.clone(),
+        )?;
         let pkg_manifest =
             xwin::manifest::get_package_manifest(ctx, &manifest, manifest_pb.clone())?;
         manifest_pb.finish_with_message("📥 downloaded");
